summaryrefslogtreecommitdiff
path: root/pack-check.c
diff options
context:
space:
mode:
Diffstat (limited to 'pack-check.c')
-rw-r--r--pack-check.c49
1 files changed, 22 insertions, 27 deletions
diff --git a/pack-check.c b/pack-check.c
index e4ef71c..25104d5 100644
--- a/pack-check.c
+++ b/pack-check.c
@@ -1,17 +1,15 @@
-#include "cache.h"
+#include "git-compat-util.h"
+#include "environment.h"
+#include "hex.h"
#include "repository.h"
#include "pack.h"
-#include "pack-revindex.h"
#include "progress.h"
#include "packfile.h"
-#include "object-store.h"
+#include "object-file.h"
+#include "object-store-ll.h"
struct idx_entry {
off_t offset;
- union idx_entry_object {
- const unsigned char *hash;
- struct object_id *oid;
- } oid;
unsigned int nr;
};
@@ -43,7 +41,7 @@ int check_pack_crc(struct packed_git *p, struct pack_window **w_curs,
} while (len);
index_crc = p->index_data;
- index_crc += 2 + 256 + p->num_objects * (the_hash_algo->rawsz/4) + nr;
+ index_crc += 2 + 256 + (size_t)p->num_objects * (the_hash_algo->rawsz/4) + nr;
return data_crc != ntohl(*index_crc);
}
@@ -97,9 +95,6 @@ static int verify_packfile(struct repository *r,
entries[nr_objects].offset = pack_sig_ofs;
/* first sort entries by pack offset, since unpacking them is more efficient that way */
for (i = 0; i < nr_objects; i++) {
- entries[i].oid.hash = nth_packed_object_sha1(p, i);
- if (!entries[i].oid.hash)
- die("internal error pack-check nth-packed-object");
entries[i].offset = nth_packed_object_offset(p, i);
entries[i].nr = i;
}
@@ -107,11 +102,16 @@ static int verify_packfile(struct repository *r,
for (i = 0; i < nr_objects; i++) {
void *data;
+ struct object_id oid;
enum object_type type;
unsigned long size;
off_t curpos;
int data_valid;
+ if (nth_packed_object_id(&oid, p, entries[i].nr) < 0)
+ BUG("unable to get oid of object %lu from %s",
+ (unsigned long)entries[i].nr, p->pack_name);
+
if (p->index_version > 1) {
off_t offset = entries[i].offset;
off_t len = entries[i+1].offset - offset;
@@ -119,7 +119,7 @@ static int verify_packfile(struct repository *r,
if (check_pack_crc(p, w_curs, offset, len, nr))
err = error("index CRC mismatch for object %s "
"from %s at offset %"PRIuMAX"",
- oid_to_hex(entries[i].oid.oid),
+ oid_to_hex(&oid),
p->pack_name, (uintmax_t)offset);
}
@@ -129,7 +129,7 @@ static int verify_packfile(struct repository *r,
if (type == OBJ_BLOB && big_file_threshold <= size) {
/*
* Let check_object_signature() check it with
* the streaming interface; no point slurping
* the data in-core only to discard.
*/
@@ -142,14 +142,18 @@ static int verify_packfile(struct repository *r,
if (data_valid && !data)
err = error("cannot unpack %s from %s at offset %"PRIuMAX"",
- oid_to_hex(entries[i].oid.oid), p->pack_name,
+ oid_to_hex(&oid), p->pack_name,
(uintmax_t)entries[i].offset);
- else if (check_object_signature(r, entries[i].oid.oid, data, size, type_name(type)))
+ else if (data && check_object_signature(r, &oid, data, size,
+ type) < 0)
+ err = error("packed %s from %s is corrupt",
+ oid_to_hex(&oid), p->pack_name);
+ else if (!data && stream_object_signature(r, &oid) < 0)
err = error("packed %s from %s is corrupt",
- oid_to_hex(entries[i].oid.oid), p->pack_name);
+ oid_to_hex(&oid), p->pack_name);
else if (fn) {
int eaten = 0;
- err |= fn(entries[i].oid.oid, type, size, data, &eaten);
+ err |= fn(&oid, type, size, data, &eaten);
if (eaten)
data = NULL;
}
@@ -166,22 +170,13 @@ static int verify_packfile(struct repository *r,
int verify_pack_index(struct packed_git *p)
{
- off_t index_size;
- const unsigned char *index_base;
- git_hash_ctx ctx;
- unsigned char hash[GIT_MAX_RAWSZ];
int err = 0;
if (open_pack_index(p))
return error("packfile %s index not opened", p->pack_name);
- index_size = p->index_size;
- index_base = p->index_data;
/* Verify SHA1 sum of the index file */
- the_hash_algo->init_fn(&ctx);
- the_hash_algo->update_fn(&ctx, index_base, (unsigned int)(index_size - the_hash_algo->rawsz));
- the_hash_algo->final_fn(hash, &ctx);
- if (!hasheq(hash, index_base + index_size - the_hash_algo->rawsz))
+ if (!hashfile_checksum_valid(p->index_data, p->index_size))
err = error("Packfile index for %s hash mismatch",
p->pack_name);
return err;
@@ -129,7 +129,7 @@ static int verify_packfile(struct repository *r,
if (type == OBJ_BLOB && big_file_threshold <= size) {
/*
- * Let check_object_signature() check it with
+ * Let stream_object_signature() check it with
* the streaming interface; no point slurping
* the data in-core only to discard.